@@ -1,6 +1,6 @@
 /* Declarations for convert.c
-   Copyright (C) 2003, 2004, 2005, 2006, 2009, 2010, 2011, 2015 Free
-   Software Foundation, Inc.
+   Copyright (C) 2003-2006, 2009-2011, 2015, 2018 Free Software
+   Foundation, Inc.
 
 This file is part of GNU Wget.
 
