@@ -1,7 +1,5 @@
 /* URL handling.
-   Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-   2005, 2006, 2007, 2008, 2009, 2010, 2011, 2015 Free Software
-   Foundation, Inc.
+   Copyright (C) 1996-2011, 2015, 2018 Free Software Foundation, Inc.
 
 This file is part of GNU Wget.
 
@@ -459,7 +457,7 @@ url_scheme (const char *url)
   int i;
 
   for (i = 0; supported_schemes[i].leading_string; i++)
-    if (0 == strncasecmp (url, supported_schemes[i].leading_string,
+    if (0 == c_strncasecmp (url, supported_schemes[i].leading_string,
                           strlen (supported_schemes[i].leading_string)))
       {
         if (!(supported_schemes[i].flags & scm_disabled))
@@ -925,6 +923,17 @@ url_parse (const char *url, int *error, struct iri *iri, bool percent_encode)
       url_unescape (u->host);
       host_modified = true;
 
+      /* check for invalid control characters in host name */
+      for (p = u->host; *p; p++)
+        {
+          if (c_iscntrl(*p))
+            {
+              url_free(u);
+              error_code = PE_INVALID_HOST_NAME;
+              goto error;
+            }
+        }
+
       /* Apply IDNA regardless of iri->utf8_encode status */
       if (opt.enable_iri && iri)
         {
@@ -933,7 +942,6 @@ url_parse (const char *url, int *error, struct iri *iri, bool percent_encode)
             {
               xfree (u->host);
               u->host = new;
-              u->idn_allocated = true;
               host_modified = true;
             }
         }
@@ -1212,12 +1220,7 @@ url_free (struct url *url)
 {
   if (url)
     {
-      if (url->idn_allocated) {
-        idn2_free (url->host);      /* A dummy if !defined(ENABLE_IRI) */
-        url->host = NULL;
-      }
-      else
-        xfree (url->host);
+      xfree (url->host);
 
       xfree (url->path);
       xfree (url->url);
@@ -1284,7 +1287,7 @@ mkalldirs (const char *path)
     }
   res = make_directory (t);
   if (res != 0)
-    logprintf (LOG_NOTQUIET, "%s: %s", t, strerror (errno));
+    logprintf (LOG_NOTQUIET, "%s: %s\n", t, strerror (errno));
   xfree (t);
   return res;
 }
@@ -1544,6 +1547,7 @@ append_uri_pathel (const char *b, const char *e, bool escaped,
   append_null (dest);
 }
 
+#ifdef HAVE_ICONV
 static char *
 convert_fname (char *fname)
 {
@@ -1562,9 +1566,9 @@ convert_fname (char *fname)
     to_encoding = nl_langinfo (CODESET);
 
   cd = iconv_open (to_encoding, from_encoding);
-  if (cd == (iconv_t)(-1))
-    logprintf (LOG_VERBOSE, _("Conversion from %s to %s isn't supported\n"),
-	       quote (from_encoding), quote (to_encoding));
+  if (cd == (iconv_t) (-1))
+    logprintf (LOG_VERBOSE, _ ("Conversion from %s to %s isn't supported\n"),
+               quote (from_encoding), quote (to_encoding));
   else
     {
       inlen = strlen (fname);
@@ -1573,50 +1577,62 @@ convert_fname (char *fname)
       done = 0;
 
       for (;;)
-	{
-	  if (iconv (cd, (ICONV_CONST char **) &fname, &inlen, &s, &outlen) != (size_t)(-1)
-	      && iconv (cd, NULL, NULL, &s, &outlen) != (size_t)(-1))
-	    {
-	      *(converted_fname + len - outlen - done) = '\0';
-	      iconv_close(cd);
-	      DEBUGP (("Converted file name '%s' (%s) -> '%s' (%s)\n",
-		       orig_fname, from_encoding, converted_fname, to_encoding));
-	      xfree (orig_fname);
-	      return converted_fname;
-	    }
+        {
+          errno = 0;
+          if (iconv (cd, (ICONV_CONST char **) &fname, &inlen, &s, &outlen) == 0
+              && iconv (cd, NULL, NULL, &s, &outlen) == 0)
+            {
+              *(converted_fname + len - outlen - done) = '\0';
+              iconv_close (cd);
+              DEBUGP (("Converted file name '%s' (%s) -> '%s' (%s)\n",
+                       orig_fname, from_encoding, converted_fname, to_encoding));
+              xfree (orig_fname);
+              return converted_fname;
+            }
 
-	  /* Incomplete or invalid multibyte sequence */
-	  if (errno == EINVAL || errno == EILSEQ)
-	    {
-	      logprintf (LOG_VERBOSE,
-			 _("Incomplete or invalid multibyte sequence encountered\n"));
-	      xfree (converted_fname);
-	      converted_fname = (char *)orig_fname;
-	      break;
-	    }
-	  else if (errno == E2BIG) /* Output buffer full */
-	    {
-	      done = len;
-	      len = outlen = done + inlen * 2;
-	      converted_fname = xrealloc (converted_fname, outlen + 1);
-	      s = converted_fname + done;
-	    }
-	  else /* Weird, we got an unspecified error */
-	    {
-	      logprintf (LOG_VERBOSE, _("Unhandled errno %d\n"), errno);
-	      xfree (converted_fname);
-	      converted_fname = (char *)orig_fname;
-	      break;
-	    }
-	}
+          /* Incomplete or invalid multibyte sequence */
+          if (errno == EINVAL || errno == EILSEQ || errno == 0)
+            {
+              if (errno)
+                logprintf (LOG_VERBOSE,
+                           _ ("Incomplete or invalid multibyte sequence encountered\n"));
+              else
+                logprintf (LOG_VERBOSE,
+                           _ ("Unconvertable multibyte sequence encountered\n"));
+              xfree (converted_fname);
+              converted_fname = (char *) orig_fname;
+              break;
+            }
+          else if (errno == E2BIG) /* Output buffer full */
+            {
+              done = len;
+              len = outlen = done + inlen * 2;
+              converted_fname = xrealloc (converted_fname, outlen + 1);
+              s = converted_fname + done;
+            }
+          else /* Weird, we got an unspecified error */
+            {
+              logprintf (LOG_VERBOSE, _ ("Unhandled errno %d\n"), errno);
+              xfree (converted_fname);
+              converted_fname = (char *) orig_fname;
+              break;
+            }
+        }
       DEBUGP (("Failed to convert file name '%s' (%s) -> '?' (%s)\n",
-	       orig_fname, from_encoding, to_encoding));
+               orig_fname, from_encoding, to_encoding));
     }
 
     iconv_close(cd);
 
   return converted_fname;
 }
+#else
+static char *
+convert_fname (char *fname)
+{
+  return fname;
+}
+#endif
 
 /* Append to DEST the directory structure that corresponds the
    directory part of URL's path.  For example, if the URL is
@@ -1690,35 +1706,43 @@ url_file_name (const struct url *u, char *replaced_filename)
   /* If "dirstruct" is turned on (typically the case with -r), add
      the host and port (unless those have been turned off) and
      directory structure.  */
+  /* All safe remote chars are unescaped and stored in temp_fnres,
+     then converted to local and appended to fnres.
+     Internationalized URL/IDN will produce punycode to lookup IP from DNS:
+     https://en.wikipedia.org/wiki/URL
+     https://en.wikipedia.org/wiki/Internationalized_domain_name
+     Non-ASCII code chars in the path:
+     https://en.wikipedia.org/wiki/List_of_Unicode_characters
+     https://en.wikipedia.org/wiki/List_of_writing_systems */
   if (opt.dirstruct)
     {
       if (opt.protocol_directories)
         {
-          if (fnres.tail)
-            append_char ('/', &fnres);
-          append_string (supported_schemes[u->scheme].name, &fnres);
+          if (temp_fnres.tail)
+            append_char ('/', &temp_fnres);
+          append_string (supported_schemes[u->scheme].name, &temp_fnres);
         }
       if (opt.add_hostdir)
         {
-          if (fnres.tail)
-            append_char ('/', &fnres);
+          if (temp_fnres.tail)
+            append_char ('/', &temp_fnres);
           if (0 != strcmp (u->host, ".."))
-            append_string (u->host, &fnres);
+            append_string (u->host, &temp_fnres);
           else
             /* Host name can come from the network; malicious DNS may
                allow ".." to be resolved, causing us to write to
                "../<file>".  Defang such host names.  */
-            append_string ("%2E%2E", &fnres);
+            append_string ("%2E%2E", &temp_fnres);
           if (u->port != scheme_default_port (u->scheme))
             {
               char portstr[24];
               number_to_string (portstr, u->port);
-              append_char (FN_PORT_SEP, &fnres);
-              append_string (portstr, &fnres);
+              append_char (FN_PORT_SEP, &temp_fnres);
+              append_string (portstr, &temp_fnres);
             }
         }
 
-      append_dir_structure (u, &fnres);
+      append_dir_structure (u, &temp_fnres);
     }
 
   if (!replaced_filename)
@@ -1739,12 +1763,23 @@ url_file_name (const struct url *u, char *replaced_filename)
       fname_len_check = strdupdelim (u_file, u_file + strlen (u_file));
     }
 
+  if (temp_fnres.tail)
+    append_char ('/', &temp_fnres);
+
   append_uri_pathel (fname_len_check,
-    fname_len_check + strlen (fname_len_check), false, &temp_fnres);
+    fname_len_check + strlen (fname_len_check), true, &temp_fnres);
 
   /* Zero-terminate the temporary file name. */
   append_char ('\0', &temp_fnres);
 
+  /* convert all remote chars before length check and appending to local path */
+  fname = convert_fname (temp_fnres.base);
+  temp_fnres.base = NULL;
+  temp_fnres.size = 0;
+  temp_fnres.tail = 0;
+  append_string (fname, &temp_fnres);
+  xfree (fname);
+
   /* Check that the length of the file name is acceptable. */
 #ifdef WINDOWS
   if (MAX_PATH > (fnres.tail + CHOMP_BUFFER + 2))
@@ -1793,8 +1828,6 @@ url_file_name (const struct url *u, char *replaced_filename)
 
   xfree (temp_fnres.base);
 
-  fname = convert_fname (fname);
-
   /* Check the cases in which the unique extensions are not used:
      1) Clobbering is turned off (-nc).
      2) Retrieval with regetting.
@@ -1806,7 +1839,7 @@ url_file_name (const struct url *u, char *replaced_filename)
      directory (see `mkalldirs' for explanation).  */
 
   if (ALLOW_CLOBBER
-      && !(file_exists_p (fname) && !file_non_directory_p (fname)))
+      && !(file_exists_p (fname, NULL) && !file_non_directory_p (fname)))
     {
       unique = fname;
     }
